package com.persagy.proxy.adm.handler;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.alibaba.excel.metadata.CellData;
import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.proxy.adm.model.AdmRelationObject;
import com.persagy.proxy.adm.strategy.QueryRelationObjectContext;

import cn.hutool.core.collection.CollectionUtil;
import lombok.extern.slf4j.Slf4j;

/**
 * 传感器数据导入， 此类不能被spring管理，要每次读取excel都要new
 * 
 * @version v1.0.0
 * @company persagy
 * @author zhangqiankun
 * @date 2020-11-25 19:52:37
 */
@Slf4j
@SuppressWarnings("unchecked")
public class RelationImportExcelDataHandler extends AnalysisEventListener<AdmRelationObject> {
	
	private final String groupCode;
	
	private final String projectId;
	
	private final String relType;
	
	private String code;
	
	private final ObjectMapper objectMapper = SpringHelper.getBean(ObjectMapper.class);
	
	private final QueryRelationObjectContext queryRelationObjectContext;
	
	/** 失败信息描述*/
	private Map<Integer, String> failureMap = new HashMap<Integer, String>(256);
	
	private String graphId;		// 图类型ID
	
	private int failureNum = 0;
	
	private int successNum = 0;
	

	/**
	 * 上传的数据
	 */
	private List<ObjectNode> relationObjects = new ArrayList<ObjectNode>();
	
	public RelationImportExcelDataHandler(String groupCode, String projectId, String relType, QueryRelationObjectContext queryRelationObjectContext) {
		this.relType = relType;
		this.groupCode = groupCode;
		this.projectId = projectId;
		this.queryRelationObjectContext = queryRelationObjectContext;
	}
	
	/**
	 * 每一条数据解析都会来调用
	 * 
	 * @param data one row value {@link AnalysisContext#readRowHolder()}
	 * @param context
	 */
	@Override
	public void invoke(AdmRelationObject data, AnalysisContext context) {
		// 数据不可为空校验
		int rowIndex = context.readRowHolder().getRowIndex();
		if (rowIndex == 2 && code == null) {
			CellData<String> cell = (CellData<String>)context.readRowHolder().getCellMap().get(1);
			code = cell.getStringValue();
		}
		// 数据校验
		ObjectNode objectNode = this.objectMapper.createObjectNode();
		String checkResult = this.queryRelationObjectContext.checkRelationObject(objectNode, data, groupCode, projectId, relType, code);
		if (checkResult != null) {
			this.failureMap.put(rowIndex, checkResult);
			return;
		}
		this.graphId = objectNode.get("graphId") == null ? graphId : objectNode.get("graphId").asText();
		
		// 批量插入
		relationObjects.add(objectNode);
		if (relationObjects.size() >= 1000) {
			boolean saveRelationObjects = this.queryRelationObjectContext.saveRelationObjects(relationObjects, groupCode, projectId, relType);
			if (saveRelationObjects) {
				successNum += relationObjects.size();
			} else {
				failureNum += relationObjects.size();
			}
			relationObjects.clear();
		}
	}

	/**
	 * 所有数据解析完成了都会来调用，在这里进行数据存储，可确保最后遗留的数据也被存储到数据库
	 * 
	 * @param context
	 */
	@Override
	public void doAfterAllAnalysed(AnalysisContext context) {
		if (CollectionUtil.isNotEmpty(relationObjects)) {
			boolean saveRelationObjects = this.queryRelationObjectContext.saveRelationObjects(relationObjects, groupCode, projectId, relType);
			if (saveRelationObjects) {
				successNum += relationObjects.size();
			} else {
				failureNum += relationObjects.size();
			}
			relationObjects.clear();
		}
		log.info("集团: {}，项目: {}，类型：{}，关系数据导入成功，", groupCode, projectId, relType);
	}

	/**
	 * 数据写入完成后，可以通过此方法获取部分信息
	 * 
	 * @return
	 */
	public JSONObject afterWriteObject() {
		JSONObject result = new JSONObject();
		result.put("relationType", graphId);
		result.put("successCount", successNum);
		result.put("failCount", failureNum);
		String state = null;
        if (successNum > 0 && failureNum > 0) {
        	state = "1";
        } else if (successNum > 0 && failureNum == 0) {
            state = "0";
        } else {
            state = "2";
        }
        result.put("state", state);
		return result;
	}
	
}